# TubeMakerPlugin
# Initial code generated by XSI SDK Wizard


#Note:  All text beginning with a '#' character represents 
#		a comment in Python, and has no affect on how the
#		code is interpreted by XSI


#When you use Python, you can import definitions of 
#constant values that are used with XSI's scripting
#interface. This makes it a lot easier to work with
import win32com.client
from win32com.client import constants


#-------------------------------------------------------------------
#-------------------------------------------------------------------


#This is the first function that XSI looks at when loading
#a potential plugin. It contains details about who wrote it,
#support information, as well as the important declarations
#on what the plugin contains.

def XSILoadPlugin( in_reg ):
	in_reg.Author = "Andy Nicholas"
	in_reg.Name = "TCT_TubeMaker"
	in_reg.Email = ""
	in_reg.URL = ""
	in_reg.Major = 1
	in_reg.Minor = 0

	#Here we're registering a custom property
	in_reg.RegisterProperty("TubeMaker")
	
	#We also register a menu with XSI to create the pipe maker custom property and launch the tool
	in_reg.RegisterMenu(constants.siMenuTbGetPrimitivePolygonMeshID,"TubeMaker_Menu",False,False)

	return True




#-------------------------------------------------------------------
#Custom Property:
#	TubeMaker

#Function:
#	Stores the parameters needed by the tool, and provides a user interface
#

#	All custom property functions start with the name of the custom property,
#	are followed by an underscore ('_'), and then have the name of the callback.
#	A callback is just a special type of function that XSI calls at certain times.
#	So, for example, the 'Define' callback shown below is called when XSI needs to
#	know the parameters that are stored in the custom property.

#	Most other callbacks in this file deal with the user interface, and typically
#	get called when a user clicks a button or changes a value. 


#The Define callback
#Is used by XSI to find out about the parameters that belong to the custom property
def TubeMaker_Define( ctxt ):
	oCustomProperty = ctxt.Source
	
	#Here we just add each parameter one by one, specifying things like what
	#type of data it is, what range of values it's allowed to have, etc.
	oCustomProperty.AddParameter2("replacemode",constants.siBool,False,None,None,None,None,0,constants.siPersistable)
	oCustomProperty.AddParameter2("cursel",constants.siString,"",None,None,None,None,0,constants.siPersistable|constants.siReadOnly)
	oCustomProperty.AddParameter2("roll",constants.siFloat,0,-360,360,-180,180,0,constants.siPersistable)
	oCustomProperty.AddParameter2("tubepath",constants.siString,"",None,None,None,None,0,constants.siPersistable)	
	return True



#The DefineLayout callback
#XSI uses this to find out how the parameters are arranged on the custom property and
#how they are presented. We can also add buttons and other controls to the interface.
#Spacing between controls is achieved by using rows and groups to separate them.
def TubeMaker_DefineLayout( ctxt ):
	oLayout = ctxt.Source
	oLayout.Clear()
	oLayout.AddTab("Create")
	
	#This is where we add the buttons for placing the pipes
	oLayout.AddGroup("Add Tube")

	oLayout.AddRow()
	oLayout.AddGroup("",False,33)	
	oLayout.EndGroup()
	
	oLayout.AddGroup("",False,33)
	oLayout.AddButton("Up_45","    Up 45      ")
	oLayout.EndGroup()

	oLayout.AddGroup("",False,33)	
	oLayout.EndGroup()
	oLayout.EndRow()
	
	oLayout.AddRow()
	oLayout.AddGroup("",False,33)
	oLayout.AddButton("Left_45","   Left 45   ")
	oLayout.AddButton("Left_90","   Left 90   ")	
	oLayout.EndGroup()
	
	oLayout.AddGroup("",False,33)
	oLayout.AddButton("Fwd_Short","  Fwd Short ")
	oLayout.AddButton("Fwd_Long","  Fwd Long  ")
	oLayout.EndGroup()

	oLayout.AddGroup("",False,33)
	oLayout.AddButton("Right_45", "  Right 45  ")
	oLayout.AddButton("Right_90", "  Right 90  ")
	oLayout.EndGroup()
	oLayout.EndRow()

	oLayout.AddRow()
	oLayout.AddGroup("",False,33)
	oLayout.EndGroup()		

	oLayout.AddGroup("",False,33)
	oLayout.AddButton("Down_45", "   Down 45   ")
	oLayout.EndGroup()

	oLayout.AddGroup("",False,33)
	oLayout.EndGroup()
	oLayout.EndRow()
	
	oLayout.EndGroup()
	
	#Replace mode toggle check box
	oLayout.AddGroup()
	oLayout.AddItem("replacemode","Replace Mode")
	oLayout.EndGroup()
	
	#Selection control
	oLayout.AddGroup("Selection")
	oLayout.AddItem("cursel","Current Selection")
	oLayout.AddRow()	
	oLayout.AddStaticText(" ")
	oLayout.AddButton("first", " [< ")
	oLayout.AddStaticText(" ")
	oLayout.AddButton("prev", " < ")
	oLayout.AddButton("next", " > ")
	oLayout.AddStaticText(" ")
	oLayout.AddButton("last", " >] ")
	oLayout.EndRow()			
	oLayout.AddRow()
	oLayout.AddStaticText(" ")	
	oLayout.AddButton("show", " Show ")
	oLayout.AddButton("clear", "  Clear  ")
	oLayout.AddButton("set", "  Set  ")
	oLayout.AddButton("delete"," Delete ")
	oLayout.EndRow()	
	oLayout.EndGroup()
	
	#Tube angle adjustment control
	oLayout.AddGroup("Adjust")
	oLayout.AddItem("roll","Roll")	
	oLayout.EndGroup()
	
	#Finally add the parameter that stores where the tube models are located
	oLayout.AddTab("Settings")
	oLayout.AddItem("tubepath","Tube Path", constants.siControlFolder)
	
	return True

#The OnInit callback
#XSI calls this function when the property page is first created,
#and each time that the page is shown
def TubeMaker_OnInit( ):
	#Here we set a default value for the tube path to point to the
	#models directory of the current project (which is probably a good guess)
	if PPG.tubepath.Value=="":
		PPG.tubepath.Value = Application.ActiveProject.Path+"\\Models"



#Each of the following functions is a callback that is executed when a button is pressed.
#The name of each button makes up part of the function name which is how XSI knows which
#one to call.

#Each function just makes a call to the TCT_AddTube command with the appropriate arguments.
#By doing this, we can keep all the code in one place, and not have to duplicate the code
#between each of these callbacks. This is considered good practice when programming.
def TubeMaker_Up_45_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\Up45_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)

def TubeMaker_Left_90_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\Left90_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)

def TubeMaker_Left_45_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\Left45_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)

def TubeMaker_Fwd_Short_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\FwdShort_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)

def TubeMaker_Fwd_Long_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\FwdLong_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)

def TubeMaker_Right_45_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\Right45_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)
	
def TubeMaker_Right_90_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\Right90_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)
	
def TubeMaker_Down_45_OnClicked( ):
	newTube = Application.TCT_AddTube(PPG.tubepath.Value+"\\Down45_Tube.emdl", PPG.cursel.Value, PPG.replacemode.Value)
	UpdateTubeMaker(PPG.Inspected(0),newTube)



#This callback is executed whenever the Roll parameter is changed by the user.
#It allows us to find the currently selected tube and adjust the constraint accordingly.
def TubeMaker_roll_OnChanged( ):
	#Find the constraint
	cns = Application.TCTHelper_GetTubeConstraint(PPG.cursel)
	
	#If it exists, alter the Z rotation to rotate the tube
	if cns!=None:
		cns.Parameters("rotz").Value = PPG.roll.Value




#These callbacks respond to button presses to show and modify the current tube selection  
def TubeMaker_show_OnClicked( ):
	#Highlight the currently selected tube in the viewport
	ShowCurrentTubeSelection(PPG.cursel.Value)

	
def TubeMaker_clear_OnClicked( ):
	#Clear the currently selected tube (this allows the user to 
	#create a new tube network)
	PPG.cursel.Value=""


def TubeMaker_set_OnClicked( ):
	#Use the PickObject command to initiate an interactive picking session for the user
	#to select which tube they want to modify
	retcol = Application.PickObject("Select Tube","Select Tube")
	if retcol('ButtonPressed')==0:
		return

	#Make sure it's a valid tube
	obj = retcol('PickedElement')
	tubeModel = obj.Model
	if not Application.TCTHelper_IsValidTube(tubeModel.FullName):
		tubeModel=None

	#Set the current selection parameter on the property
	prop=PPG.Inspected(0)
	UpdateTubeMaker(prop, tubeModel)





#This callback responds to the user pressing the Delete button.
#It will cause the currently selected tube to be deleted
def TubeMaker_delete_OnClicked( ):
	#Make sure we have a valid selection
	if PPG.cursel.Value=="":
		return
	
	#Call the TCT_DeleteTube command to remove the tube
	newTube = Application.TCT_DeleteTube(PPG.cursel.Value)
	
	#Update the property page with a new tube selection if available
	UpdateTubeMaker(PPG.Inspected(0),newTube)



#These callbacks respond to the user clicking the buttons
#to move the current selection. This allows the user to move
#the selection along the tube network with ease.
def TubeMaker_first_OnClicked( ):
	#Get currently selected tube name
	curTubeName = PPG.cursel
	prop=PPG.Inspected(0)
	prevTubeObj = curTubeName
	oldPrevTubeObj = prevTubeObj
	
	#Search backwards through the connected tubes until we can't find any more,
	#at which point we know we must be at the beginning as required
	while(prevTubeObj!=None):
		oldPrevTubeObj=prevTubeObj	
		prevTubeObj = Application.TCTHelper_GetPreviousTube(prevTubeObj.FullName)
		
	UpdateTubeMaker(prop, oldPrevTubeObj)
	
	
def TubeMaker_prev_OnClicked( ):
	#Get currently selected tube name
	curTubeName = PPG.cursel
	prop=PPG.Inspected(0)
	
	#Get the previous tube
	prevTubeObj = Application.TCTHelper_GetPreviousTube(curTubeName.FullName)
	if prevTubeObj==None:
		return
	UpdateTubeMaker(prop, prevTubeObj)

def TubeMaker_next_OnClicked( ):
	#Get currently selected tube name
	curTubeName = PPG.cursel
	prop=PPG.Inspected(0)
	
	#Get the next tube
	nextTubeObj = Application.TCTHelper_GetNextTube(curTubeName.FullName)
	if nextTubeObj==None:
		return
	UpdateTubeMaker(prop, nextTubeObj)
	
def TubeMaker_last_OnClicked( ):
	#Get currently selected tube name
	curTubeName = PPG.cursel
	prop=PPG.Inspected(0)
	nextTubeObj = curTubeName
	oldNextTubeObj = nextTubeObj
	
	#Search forwards through the connected tubes until we can't find any more,
	#at which point we know we must be at the end as required
	while(nextTubeObj!=None):
		oldNextTubeObj=nextTubeObj
		nextTubeObj = Application.TCTHelper_GetNextTube(nextTubeObj.FullName)		
	UpdateTubeMaker(prop, oldNextTubeObj)	






#These remaining two functions are not callbacks, but are helper functions.
#They are used to do common tasks and so they prevent repetition of code


#This helper function updates the current selection, and shows it in the viewport.
#It also makes sure that the roll parameter is showing the correct value
def UpdateTubeMaker(prop, newTubeObj):
	if newTubeObj!=None:
		prop.Parameters("cursel").Value=newTubeObj.FullName

		cns = Application.TCTHelper_GetTubeConstraint(newTubeObj.FullName)
		if cns!=None:
			prop.Parameters("roll").Value = cns.Parameters("rotz").Value
	
		ShowCurrentTubeSelection(newTubeObj.FullName)
	else:
		prop.Parameters("cursel").Value=""
	
#This helper function simply shows the currently selected tube in 
#the viewport by selecting the tube's mesh objects
def ShowCurrentTubeSelection(selTubeName):
	meshObj = Application.TCTHelper_GetTubeMeshObj(selTubeName)
	if meshObj!=None:
		Application.Selection.SetAsText(meshObj.FullName)








#-------------------------------------------------------------------

#Menu registration
def TubeMaker_Menu_Init( ctxt ):
	oMenu = ctxt.Source
	#This line registers a callback function that is called whenever the
	#user selects the menu item. This is a very quick and convenient way
	#of creating some sort of menu driven action.
	oMenu.AddCallbackItem("Tube Maker","OnTubeMakerMenuClicked")
	return True



#Menu callback function
#This function creates a special view to show the Tube Maker property page.
#Using a view has the advantage of being able to specify a width, height, and
#position to the window, and so gives us good control over the layout.
def OnTubeMakerMenuClicked( ctxt ):

	#Try and find the Tube Maker Custom Property
	prop = Application.ActiveSceneRoot.Properties("Tube_Maker")

	#Create the Custom Property if it doesn't already exist
	if prop==None:
		prop = Application.ActiveSceneRoot.AddProperty("TubeMaker",False,"Tube_Maker")
	
	#Create a new view
	activeLayout = Application.Desktop.ActiveLayout;
	newView = activeLayout.CreateView( "Property Panel", "TubeMaker" );
	
	#Adjust the size and position
	newView.BeginEdit();
	newView.Move( 120, 75 );
	newView.Resize( 300, 355 );
	
	#Tell the view to show the Tube Maker property
	newView.SetAttributeValue( "targetcontent", prop.FullName );
	newView.EndEdit();		

	return True

