# TCT_TubeTools
# Initial code generated by XSI SDK Wizard

#Note:  All text beginning with a '#' character represents 
#		a comment in Python, and has no affect on how the
#		code is interpreted by XSI


#When you use Python, you can import definitions of 
#constant values that are used with XSI's scripting
#interface. This makes it a lot easier to work with.
import win32com.client
from win32com.client import constants


#-------------------------------------------------------------------
#-------------------------------------------------------------------


#This is the first function that XSI looks at when loading
#a potential plugin. It contains details about who wrote it,
#support information, as well as the important declarations
#on what the plugin contains.

def XSILoadPlugin( in_reg ):
	in_reg.Author = "Andy Nicholas"
	in_reg.Name = "TCT_TubeTools"
	in_reg.Email = ""
	in_reg.URL = ""
	in_reg.Major = 1
	in_reg.Minor = 0

	in_reg.RegisterCommand("TCT_CreateTube_GUI","TCT_CreateTube_GUI")
	in_reg.RegisterCommand("TCT_CreateTube","TCT_CreateTube")	
	in_reg.RegisterCommand("TCT_CreateTubeSet_GUI","TCT_CreateTubeSet_GUI")
	in_reg.RegisterCommand("TCT_CreateTubeSet","TCT_CreateTubeSet")

	in_reg.RegisterMenu(constants.siMenuTbGetPrimitivePolygonMeshID,"TCT_CreateTube_Menu",False,False)

	return True
	
	
	

#-------------------------------------------------------------------
#Menu registration
def TCT_CreateTube_Menu_Init( ctxt ):	
	oMenu = ctxt.Source
	oMenu.AddCommandItem("Tube","TCT_CreateTube_GUI")
	oMenu.AddCommandItem("Tube Set","TCT_CreateTubeSet_GUI")
	return True


#-------------------------------------------------------------------
#Command:
#	TCT_CreateTube_GUI

#Parameters:
#	None
	
#Return Value:	
#	Reference to the model containing the tube

#Function:
#	Provides user interface for TCT_CreateTube command


#Command Registration
def TCT_CreateTube_GUI_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

#Command Definition	
def TCT_CreateTube_GUI_Execute():
	#Create a temporary property page using the XSIFactory object
	prop = XSIFactory.CreateObject("CustomProperty")
	
	#Add the various parameters to the property page
	prop.Name="Tube_Parameters"
	prop.AddParameter2("tubename",constants.siString,"tube",None,None,None,None,constants.siClassifUnknown,constants.siPersistable)
	prop.AddParameter2("angle",constants.siFloat,90,None,None,-90,90,constants.siClassifUnknown,constants.siPersistable)
	prop.AddParameter2("direction",constants.siFloat,0,None,None,-360,360,constants.siClassifUnknown,constants.siPersistable)
	prop.AddParameter2("length",constants.siFloat,1,0.0001,None,0,20,constants.siClassifUnknown,constants.siPersistable)
	prop.AddParameter2("diameter",constants.siFloat,0.25,0.0001,None,0,20,constants.siClassifUnknown,constants.siPersistable)
	prop.AddParameter2("segs",constants.siInt4,5,1,None,1,20,constants.siClassifUnknown,constants.siPersistable)
	
	#The layout object tells XSI which parameters to show, and
	#where to place them.
	layout = prop.PPGLayout
	layout.Clear()
	layout.AddItem("tubename","Tube Name")
	layout.AddGroup("Bend")
	layout.AddItem("angle","Angle")
	layout.AddItem("direction","Direction")
	layout.EndGroup()
	
	layout.AddGroup("Geometry")
	layout.AddItem("length","Length")	
	layout.AddItem("diameter","Diameter")	
	layout.AddItem("segs","Segments")	
	layout.EndGroup()
	
	#Show the property page to the user
	hasCancelled = Application.InspectObj(prop,"","Create Tube",constants.siModal,False)
	
	if hasCancelled:
		return None
	
	#If the user hasn't press cancel, then go ahead and call
	#the TCT_CreateTube command
	params = prop.Parameters
	return Application.TCT_CreateTube(params("tubename").Value,
									  params("angle").Value,
									  params("direction").Value,
									  params("length").Value,
									  params("diameter").Value,
									  params("segs").Value)
	

#-------------------------------------------------------------------
#Command:
#	TCT_CreateTube

#Parameters:
#	tubename	Name of the tube model to create
#	angle		Bend angle of the tube (degrees)
#	direction	Roll angle of the tube (degrees)
#	length		Length of tube
#	diameter	Diameter of tube
#	segs		Number of segments in tube
	
#Return Value:	
#	Reference to the model containing the tube

#Function:
#	Creates basic tube geometry. The command puts the mesh inside a
#	newly created model and places a Null object at the end of the tube.
	

#Command Registration
def TCT_CreateTube_Init( ctxt ):	
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubename",constants.siArgumentInput,"tube")
	oArgs.Add("angle",constants.siArgumentInput,90.0)
	oArgs.Add("direction",constants.siArgumentInput,0.0)
	oArgs.Add("length",constants.siArgumentInput,1.0)
	oArgs.Add("diameter",constants.siArgumentInput,0.25)	
	oArgs.Add("segs",constants.siArgumentInput,5)	
	return True

#Command Definition
def TCT_CreateTube_Execute(tubename, angle, direction, length, diameter, segs):


	#Make sure that any PPGs (property pages) don't appear when we create objects
	#otherwise, the user has to spend a lot of time closing them later
	#(We remember to turn this preference back on at the end of the command)
	autoinspect = Application.GetValue("preferences.Interaction.autoinspect")
	Application.SetValue("preferences.Interaction.autoinspect", 0, "")

	#Store current selection (as it allows us to restore it later) and clear it
	#to make sure we don't have any unintended consequences
	origSelection = Application.Selection.GetAsText()	
	Application.Selection.Clear()
	
	#First thing to do is to create the model. We place objects inside
	#it as we create them (when possible) to prevent any naming clashes.
	#Otherwise XSI will change names to some other name if it happens to
	#match another object in the scene. This would cause problems later on.	
	retcol = Application.CreateModel("", tubename)
	tubeModel = retcol(0)	
	
	#Create a list of points to create a curve from
	pnts=["(0,0,"+str(length*x/float(segs))+")" for x in xrange(0,segs+1)]	
		
	#Create the curve and move it inside the model
	retcol = Application.CreateCurve(3, 1, ",".join(pnts), False)
	mainCurve = retcol(0)
	Application.CopyPaste(mainCurve, "", tubeModel, 1)		

	#Create a null object inside the model to place at the start of the tube
	#This is what we constrain to other tubes
	curveStartObj = Application.GetPrim("Null", "Tube_Start", tubeModel.FullName)
	Application.SetValue(curveStartObj.FullName+".null.size", 0.1, "")
		
	#Create a null object inside the model to place at the end of the tube
	#This lets us know where to attach other tubes later on
	curveEndObj = Application.GetPrim("Null", "Tube_End", tubeModel.FullName)
	Application.SetValue(curveEndObj.FullName+".null.size", 0.1, "")
	
	#Constrain the curveEndObj null to the end of the curve
	#This makes sure that it faces in the correct direction
	#when we bend the tube
	Application.ApplyCns("Path", curveEndObj, mainCurve, "")
	Application.SetValue(curveEndObj.FullName+".kine.pathcns.perc", 100, "")
	Application.SetValue(curveEndObj.FullName+".kine.pathcns.tangent", 1, "")
	Application.SetValue(curveEndObj.FullName+".kine.pathcns.dirx", 0, "")
	Application.SetValue(curveEndObj.FullName+".kine.pathcns.diry", 0, "")
	Application.SetValue(curveEndObj.FullName+".kine.pathcns.dirz", 1, "")
	Application.SetValue(curveEndObj.FullName+".kine.pathcns.upvct_active", 1, "")
	
	#Bend the tube to the correct angle, but don't bother if angle is zero
	if angle!=0:
		Application.ApplyOp("Bend", mainCurve.FullName, 3, "siPersistentOperation", "", 0)
		Application.SetValue(mainCurve.FullName+".crvlist.bendop.axis", 2, "")
		Application.SetValue(mainCurve.FullName+".bendop.dir", direction, "")
		Application.SetValue(mainCurve.FullName+".bendop.angle", angle, "")
		Application.SetValue(mainCurve.FullName+".bendop.radius", abs(length/(angle*3.14159265358979/180.0)), "")
		
	#Create a NURBS circle and extrude it along the main curve
	circleCurve = Application.CreatePrim("Circle", "NurbsCurve", "", tubeModel.FullName)
	Application.SetValue(circleCurve.FullName+".circle.radius", diameter*0.5, "")
	opcol = Application.ApplyGenOp("Extrusion", "", circleCurve.FullName+";"+mainCurve.FullName, 3, "siPersistentOperation", "siKeepGenOpInputs", "")
	
	#Obtain a reference to the newly created mesh.
	#We can get this reference from the newly created extrusion operator
	tubeMesh = opcol(0).Parent.Parent3DObject;	
	Application.CopyPaste(tubeMesh, "", curveStartObj, 1)
	tubeMesh.Name="Tube_Mesh"
	
	Application.CopyPaste(curveEndObj, "", curveStartObj, 1)
		
	#Freeze the extrusion operator on the tube, and delete the temporary curves
	Application.FreezeObj(tubeMesh, "", "")

	Application.DeleteObj(circleCurve.FullName+","+mainCurve.FullName)
	
	#Restore the original selection for neatness
	if origSelection!="":
		Application.Selection.SetAsText(origSelection)

	
	#Restore the original user settings for the PPG creation popup behaviour	
	Application.SetValue("preferences.Interaction.autoinspect", autoinspect, "")	
	
	#Return the final tube model
	return tubeModel	
	
	
	
#-------------------------------------------------------------------
#Command:
#	TCT_CreateTubeSet_GUI

#Parameters:
#	None
	
#Return Value:	
#	None

#Function:
#	Could provide user interface for TCT_CreateTubeSet command
#	if it needs one in the future. This GUI command exists
#	to preserve the convention of the user interface calling
#	the GUI version of the command, rather than the calling 
#	the command directly


#Command Registration
def TCT_CreateTubeSet_GUI_Init( ctxt ):
	#TCT_CreateTubeSet command registration
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = False
	oCmd.SetFlag(constants.siNoLogging)

#Command Definition
def TCT_CreateTubeSet_GUI_Execute():	
	Application.TCT_CreateTubeSet()

	

#-------------------------------------------------------------------
#Command:
#	TCT_CreateTubeSet

#Parameters:
#	None
	
#Return Value:	
#	None

#Function:
#	Creates a set of 6 commonly used tubes


#Command Registration	
def TCT_CreateTubeSet_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = False
	oCmd.SetFlag(constants.siNoLogging)
	
#Command Definition
def TCT_CreateTubeSet_Execute():	
	Application.TCT_CreateTube("Left90_Tube", 90, -90, 5, 1, 5)
	Application.TCT_CreateTube("Left45_Tube", 45, -90, 5, 1, 5)
	Application.TCT_CreateTube("FwdShort_Tube", 0, 0, 2.5, 1, 1)	
	Application.TCT_CreateTube("FwdLong_Tube", 0, 0, 5, 1, 1)	
	Application.TCT_CreateTube("Right90_Tube", 90, 90, 5, 1, 5)
	Application.TCT_CreateTube("Right45_Tube", 45, 90, 5, 1, 5)	
	Application.TCT_CreateTube("Up45_Tube", 45, 0, 5, 1, 5)
	Application.TCT_CreateTube("Down45_Tube", 45, 180, 5, 1, 5)

	
	
